#include <QApplication>
#include <QDesktopWidget>
#include <QVBoxLayout>
#include <QTableWidget>
#include <QPushButton>
#include <QHBoxLayout>
#include <QStringList>
#include <QHeaderView>

#include "choiceqdlg.h"
#include "tournament.h"
#include "quest.h"

#include "images/button_cancel.xpm"
#include "images/button_ok.xpm"
/**
* \brief Constructor
* \param aTourn -   
* \param aShowRems - true      
*/
QE_ChoiceQuestDlg::QE_ChoiceQuestDlg(QWidget *parent, QE_Tournament &aTourn,
	   bool aShowRems): QDialog(parent)
{
	QStringList headers(tr("Text of question"));
	QVBoxLayout *topl = new QVBoxLayout(this);
	table = new QTableWidget(aTourn.countQuests(), 1 + aShowRems, this);
	if (aShowRems)
		headers << tr("Remark of question");
	table->setHorizontalHeaderLabels(headers);
	//table->setDragEnabled(true);
	topl->addWidget(table);
	
	QHBoxLayout *hb = new QHBoxLayout;
	hb->setSpacing(2);
	topl->addItem(hb);
	
	QPushButton *pb = new QPushButton(QIcon(button_ok), tr("Ok"), this);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotOk()));
	pb = new QPushButton(QIcon(button_cancel), tr("&Cancel"), this);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(reject()));

	setSizeGripEnabled(true);
	setMaximumHeight((QApplication::desktop()->height()*9)/10);
	showMaximized();
	fillTable(aTourn, aShowRems);
} /* QE_ChoiceQuestDlg::*parent, const QE_Tournament &aTourn, bool aShowRems) */

/**
* \brief   
* \param aTourn -  
*/
void
QE_ChoiceQuestDlg::fillTable(QE_Tournament &aTourn, bool aShowRems) const
{
	QStringList rowNums;
	int i, count = aTourn.countQuests();
	QTableWidgetItem *it;
	for( i = 0; i < count; ++i)
	{
		it = new QTableWidgetItem();
		it->setFlags(Qt::ItemIsUserCheckable);
		it->setCheckState(Qt::Unchecked);
		it->setData(0, aTourn.ptrQuest(i)->text());
		table->setItem(i, 0, it);
		// [    
		if (aShowRems)
		{
			it = new QTableWidgetItem(aTourn.ptrQuest(i)->remarks());
			it->setFlags(Qt::ItemIsUserCheckable);
			table->setItem(i, 1, it);
		} /* 		if (aShowRems) */
		rowNums << tr("%1").arg(i + 1);
	} /* 	for( int i = 0; i < count; ++i) */
	for( i = 0; i < count; ++i)
		table->resizeRowToContents(i);
	
	table->setVerticalHeaderLabels(rowNums);
} /* QE_ChoiceQuestDlg::fillTable(const QE_Tournament &aTourn) */


/**
* \brief slot    
*/
void
QE_ChoiceQuestDlg::slotOk()
{
	QDialog::accept();
} /* QE_ChoiceQuestDlg::slotOk() */

/**
* Resize event
*/
void
QE_ChoiceQuestDlg::resizeEvent(QResizeEvent *event)
{
	QDialog::resizeEvent(event);
	int width = table->horizontalHeader()->count() > 1 ? childrenRect().width() : 0; 
	table->horizontalHeader()->resizeSection(0, childrenRect().width() - 
    			(table->verticalHeader()->width() << 1) - (width >> 2));
	//    ,    
	if (width)
		table->horizontalHeader()->resizeSection(1, width >> 2);
} /* QE_ChoiceQuestDlg::resizeEvent(QResizeEvent *event) */

/**
* \brief     [ 
*/
QVector<int>
QE_ChoiceQuestDlg::selectedQuests() const
{
	QVector<int> quests; 
	for( int i = 0; i < table->rowCount(); ++i)
		if( table->item(i, 0)->checkState() == Qt::Checked)
			quests.append(i);
	return quests;
} /* QE_ChoiceQuestDlg::selectedQuests() const */
