#include <QMessageBox>
#include <QFile>
#include <QTextCodec>

#include "exporter.h"
#include "tournament.h"
#include "quest.h"
/**
* 
*/
QE_Exporter::QE_Exporter()
{

} /* QE_Exporter */



/**
*   
*/	
bool
QE_Exporter::QE_ExportHandler::startElement(const QString & /* namespaceURI*/,
	   										const QString & /* localName) */,
											const QString &qName,
											const QXmlAttributes &attributes)
{
	if (qName == "info")
	{
		exporter->mShortDescr = attributes.value("shortdescr");
		exporter->mLongDescr  = attributes.value("descr");
		exporter->mFileType   = attributes.value("filedescr");
		exporter->mFileExt    = attributes.value("ext");
		exporter->mCodePage   = attributes.value("codepage");
	} /* 	if (qName == "info") */
	currentText.clear();
	return true;
} /*QE_ExportHandler::startElement(const QString &namespaceURI, const QString &localName */

/**
*   
*/
bool
QE_Exporter::QE_ExportHandler::endElement(const QString & /* namespaceURI */,
	   									  const QString & /* localName */,
										  const QString &qName)
{
	if (qName == "eol")
		exporter->mEol = currentText.replace("%l", "\n");
	else if (qName == "list")
		exporter->mList = currentText;
	else if (qName == "listitem")
		exporter->mListItem = currentText;
	else if (qName == "itemnum")
		exporter->mItemNum = currentText;
	else if (qName == "quest")
		exporter->mQuest = currentText;
	else if (qName == "answer")
		exporter->mAnswer = currentText;
	else if (qName == "score")
		exporter->mScore = currentText;
	else if (qName == "author")
		exporter->mAuthor = currentText;
	else if (qName == "comments")
		exporter->mComments = currentText;
	else if (qName == "source")
		exporter->mSource = currentText;
	else if (qName == "body")
		exporter->setBody(currentText);
	return true;	
} /* QE_ExportHandler::endElement(const QString &namespaceURI, const QString &localName */

/**
*  
*/
bool
QE_Exporter::QE_ExportHandler::characters(const QString &str)
{
	currentText += str;
	return true;
} /* QE_ExportHandler::characters(const QString &str) */

/**
*  
*/
bool
QE_Exporter::QE_ExportHandler::fatalError(const QXmlParseException &exception)
{
	QMessageBox::information(0, QObject::tr("Quested templates"),
                             QObject::tr("Parse error at line %1, column %2:\n"
                                         "%3")
                             .arg(exception.lineNumber())
                             .arg(exception.columnNumber())
                             .arg(exception.message()));
    return false;
} /* 		QE_ExportHandler::fatalError(const QXmlParseException &exception) */

/**
*      
*/
QString QE_Exporter::QE_ExportHandler::errorString() const
{
	return mErrorString;
} /* 		QE_ExportHandler::errorString() const */

QE_Exporter::QE_ExportHandler::QE_ExportHandler(QE_Exporter *a_Exporter):
	exporter(a_Exporter)
{
	
} /* 		QE_ExportHandler::*a_Exporter) */

/**
*   
* \params aFileName -   
* \returns true  [, false  
*/
bool
QE_Exporter::load(const QString &aFileName)
{
	QE_Exporter::QE_ExportHandler handler(this);
	QXmlSimpleReader reader;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	
	QFile file(aFileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0, QObject::tr("quested"),
                             QObject::tr("Cannot read file %1:\n%2.")
                             .arg(aFileName)
                             .arg(file.errorString()));
        return false;
    }
    QXmlInputSource xmlInputSource(&file);
    return reader.parse(xmlInputSource);
} /* QE_Exporter::load(const QString &aFileName) */

/**
*     
* \params tournament - 
* \params resultString -   
*/

void
QE_Exporter::exportToString(QE_Tournament &tournament, QByteArray *resultString) const
{
	QTextCodec *codec = QTextCodec::codecForName(mCodePage.toAscii());
	if (codec)
	{
		QString str;
		exportToUnicodeString(tournament, &str);
		*resultString = codec->fromUnicode(str);
	} /* 	if (codec) */
} /* QE_Exporter::exportToString(Tournament *tournament, QByteArray *resultString) */

/**
*     
*     
* \params body -    
*/
void
QE_Exporter::setBody(const QString &body)
{
	int headerInd = body.indexOf("%=");
	int tourInd = body.indexOf("%#");
	int lastHeaderInd = body.lastIndexOf("%=");
	int lastTourInd = body.lastIndexOf("%#");
	
	mHeader = body.left(headerInd);
	mTourHeader = body.mid(headerInd + 2, tourInd - headerInd - 2);
	mQuestBody = body.mid(tourInd + 2, lastTourInd - tourInd - 2);
	mTourFooter = body.mid(lastTourInd + 2, lastHeaderInd - lastTourInd - 2);
	mFooter = body.mid(lastHeaderInd + 2);
} /* QE_Exporter::setBody(const QString &body) */

/**
*      
* \params tournament - 
* \params resultString -   
*/
void
QE_Exporter::exportToUnicodeString(QE_Tournament &tournament, QString *resultString,
		int aTurNum, bool aPrintHeader) const
{
	QE_Quest *qPtr;
	int i, j;
	int countTurs, countTurQuests, nQuest = 0;
	int countQuests = tournament.countQuests();
	QString tournStr;
	if (aPrintHeader)
		  tournStr = exportTourn(tournament, mHeader);
	if (tournament.ptrTournInfo()->cTurQuests)
	{
		countTurQuests = tournament.ptrTournInfo()->cTurQuests;
		countTurs = countQuests / countTurQuests;
	} /* 	if (tournament.ptrTournInfo->cTurQuests) */
	else
	{
		countTurQuests = countQuests;
		countTurs = 1;
	} /* 	else */
	// [ [    
	if( aTurNum != -1 && aTurNum < countTurs)
	{
		nQuest = aTurNum * countTurQuests;
		countTurs = aTurNum + 1;	//  1 
	} /* 	if( aTurNum != -1 && aTurNum < countTurs) */
	
	for (i = aTurNum == -1 ? 0 : aTurNum; i < countTurs; ++i)
	{
		tournStr += exportTur(tournament, mTourHeader, i + 1);
		for (j = 0; j < countTurQuests; ++j)
		{
			qPtr = tournament.ptrQuest(nQuest++);
			tournStr += exportQuest(tournament, *qPtr, j + 1);
		} /* 		for (i = 0; i < cq; ++i) */
		tournStr += exportTur(tournament, mTourFooter, i + 1);
	} /* 	for (int i = 0; i < countTurs; ++i) */
	
	if (tournament.ptrTournInfo()->cTurQuests && aTurNum == -1)
	{
		//  
		countQuests = countQuests % tournament.ptrTournInfo()->cTurQuests;
		if (countQuests)
		{
			tournStr += exportTur(tournament, mTourHeader, i + 1);
			for (j = 0; j < countQuests; ++j)
			{
				qPtr = tournament.ptrQuest(nQuest++);
				tournStr += exportQuest(tournament, *qPtr, j + 1);
			} /* 		for (i = 0; i < cq; ++i) */
			tournStr += exportTur(tournament, mTourFooter, i + 1);
		} /* 	if (countQuests) */
	} /* 	if (tournament.ptrTournInfo()->cTurQuests) */	
	if (aPrintHeader)
		tournStr += exportTourn(tournament, mFooter);
	*resultString = tournStr;
} /* QE_Exporter::exportToUnicodeString(QE_Tournament &tournament, QString *resultString) const */

/**
*    
* \params aTemplate -  
* \returns  
*/
QString
QE_Exporter::exportTourn(QE_Tournament &aTourn, const QString &aTemplate) const
{
	static const char *monthName[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
									  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
									 };
	QString tournStr(aTemplate);
	//QString str;
	QE_TournInfoData *ti = aTourn.ptrTournInfo();
	tournStr.replace("%T", ti->title);
	if (ti->month > 0 && ti->month < 12)
		tournStr.replace("%A", QString("%1-%2-%3").arg(ti->day, 2, 10, QChar('0'))
												  .arg(monthName[ti->month])
												  .arg(ti->year));
	else
		tournStr.remove("%A");
	if (ti->editors.size())
	{
		//QStringList strList;
		//foreach(QE_EditorInfo edInfo, ti->editors)
		//	strList.append(edInfo.name + ", " + edInfo.email);
		//tournStr.replace("%E", exportList(strList));
		QString edStr(ti->editors[0].name);
		if (!ti->editors[0].email.isEmpty())
		{
			edStr += ", ";
			edStr += ti->editors[0].email;
		} /* 		if (!ti->editors[0].email.isEmpty()) */
		for (int i = 1; i < ti->editors.size(); ++i)
		{
			edStr += ", ";
			edStr += ti->editors[i].name;
			if (!ti->editors[i].email.isEmpty())
			{
				edStr += ", ";
				edStr += ti->editors[i].email;
			} /* 			if (!ti->editors[i].email.isEmpty()) */
		} /* 		for (int i = 0; i < ti->editors.size(); ++i) */
		tournStr.replace("%E", edStr);
	} /* 	if (ti->editors.size()) */
	else
		tournStr.replace("%E", QObject::tr("unknown"));
	tournStr.replace("%D", ti->description);
	return tournStr;
} /* QE_Exporter::exportTourn(const QString &aTemplate) */

/**
*  
* \params aTemplate - 
* \params aTurNum -  
* \returns  
*/
QString
QE_Exporter::exportTur(QE_Tournament &/*aTourn*/, const QString &aTemplate, int aTurNum) const
{
	QString str(aTemplate);
	str.replace("%N", QString::number(aTurNum));
	return str;
} /* QE_Exporter::exportTur(const QString &aTurNum, int aTurNum) */

/**
*  
* \params aTurNum - 
* \params aQNum -  
* \returns  
*/
QString
QE_Exporter::exportQuest(QE_Tournament &/*aTourn*/, QE_Quest &qPtr, int aQNum) const
{
	QString questStr = mQuestBody;
	QString str = mQuest;
	QString str1 = qPtr.text().trimmed();
	//  
	//str1.insert(0, "   ");
	str1.replace("\n", mEol);
	str.replace("%d", QString::number(aQNum)).replace("%t", str1);
	questStr.replace("%q", str);
	str = mAnswer;
	str.replace("%t", qPtr.answer().trimmed());
	questStr.replace("%a", str);
	if (!qPtr.author().isEmpty())
	{
		str = mAuthor;
		str.replace("%t", exportList(qPtr.authors(), false));
		questStr.replace("%u", str);
	} /* 			if (!qPtr->author().isEmpty()) */
	else
		questStr.remove("%u");
	if (!qPtr.score().isEmpty())
	{
		str = mScore;
		str.replace("%t", qPtr.score().trimmed());
		questStr.replace("%o", str);
	} /* 			if (!qPtr->author().isEmpty()) */
	else
		questStr.remove("%o");
	if (!qPtr.comment().isEmpty())
	{
		str = mComments;
		str1 = qPtr.comment().trimmed();
		str1.replace("\n", mEol);
		str.replace("%t", str1);
		questStr.replace("%c", str);
	} /* 			if (!qPtr->comment().isEmpty()) */
	else
		questStr.remove("%c");
	//  
	if (qPtr.countSources())
	{
		str = mSource;
		str.replace("%t", exportList(qPtr.sources(), true));
		//str.replace("%t", qPtr.source(0));
		questStr.replace("%s", str);
	} /* 			else if (cs == 1) */
	else
		questStr.remove("%s");
	questStr.replace("%l", mEol);
	return questStr;
} /* QE_Exporter::exportQuest(const QString &aTemplate, int aQNum) */

/**
* \brief   
*       .      
*  -    
* \params aList -  
* \returns  
*/
QString
QE_Exporter::exportList(const QStringList &aList, bool isNumerate) const
{
	if (!aList.size())
		return QString();
	else if (aList.size() == 1)
		return aList[0];
	else
	{
		QString result(mList), listItems, listItemsTempl;
		QString inum;
		for (int i = 0; i < aList.size(); ++i)
		{
			listItemsTempl = mListItem;
			if (!isNumerate)
				inum = "";
			else
			{
				inum = mItemNum;
				inum = inum.replace("%n", QString::number(i + 1));
			} /* 			if (isNumerate) */
			listItems += listItemsTempl.replace("%n", inum).replace("%t", aList[i]);
		} /* 		for (int i = 0; i < aList.size(); ++i) */
		//    %l -    
		int ind = listItems.lastIndexOf("%l");
		if (ind != -1)
			listItems.remove(ind, QString("%l").size());
		return result.replace("%t", listItems).replace("%l",mEol);
	} /* 	else */
} /* QE_Exporter::exportList(const QStringList &aList) const */
