#include <QtGui>
#include <QAction>
#include <QMenu>
#include <QMessageBox>
#include <QTextEdit>
#include <QTextCursor>
#include <QToolBar>
#include <QSize>
#include <QPushButton>
#include <QLabel>
#include <QCloseEvent>

#include "mainwin.h"
#include "qecentralwidget.h"
#include "aboutdlg.h"

#include "images/filenew.xpm"
#include "images/fileopen.xpm"
#include "images/filesave.xpm"
#include "images/filesaveas.xpm"
#include "images/fileexport.xpm"
#include "images/fileprint.xpm"
#include "images/exit.xpm"
#include "images/undo.xpm"
#include "images/editcut.xpm"
#include "images/editcopy.xpm"
#include "images/editpaste.xpm"
#include "images/sort.xpm"
#include "images/help.xpm"
#include "images/add.xpm"
#include "images/quested.xpm"
#include "images/preview.xpm"
#include "images/wordprocessing.xpm"
#include "images/configure.xpm"
#include "images/idea.xpm"
#include "images/charset.xpm"
#include "images/spellcheck.xpm"
#include "images/1rightarrow.xpm"
#include "images/1leftarrow.xpm"
#include "images/2rightarrow.xpm"
#include "images/2leftarrow.xpm"
#include "images/find.xpm"
#include "images/next.xpm"

/**
* constructor
*/

MainWin::MainWin():
	QMainWindow()
{
	connect(this, SIGNAL(signalNoPreview()), this, SLOT(slotNoPreview()), Qt::QueuedConnection);
	centralWidgt = new QE_CentralWidget(this);
	setCentralWidget(centralWidgt);
	createMenuToolbar();
	lbCurNum = new QLabel(this);
	statusBar()->addWidget(lbCurNum);
	lbCountQuests = new QLabel(this);
	statusBar()->addWidget(lbCountQuests);

	connect(centralWidgt, SIGNAL(curQuestChanged(int, int)), 
			this, SLOT(slotCurQuestChanged(int, int)));

	setWindowIcon(QIcon(quested_xpm));
	restoreWinPosition();
} /* MainWin() */

/**
* creates a menu
*/

void MainWin::createMenuToolbar()
{
	//file menu&toolbar
	QToolBar *tb = new QToolBar(this);
	tb->setIconSize(QSize(16, 16));
	tb->setWindowTitle(tr("File Actions"));
	addToolBar(tb);
	QMenu *mn = new QMenu(tr("&File"), this);
	menuBar()->addMenu(mn);
	QAction *a;
	a = new QAction(QIcon(filenew_xpm), tr("&New package"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_N);
	a->setToolTip(tr("Create a new package of questions"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFileNew()));
	tb->addAction(a);
	mn->addAction(a);
	a = new QAction(QIcon(fileopen_xpm), tr("&Open..."), this);
	a->setShortcut(Qt::CTRL + Qt::Key_O);
	a->setToolTip(tr("Open package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFileOpen()));
	tb->addAction(a);
	mn->addAction(a);
	a = new QAction(QIcon(filesave_xpm), tr("&Save"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_S);
	a->setToolTip(tr("Save package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFileSave()));
	tb->addAction(a);
	mn->addAction(a);
	a = new QAction(QIcon(filesaveas_xpm), tr("Sa&ve as..."), this);
	a->setToolTip(tr("Save package with new name"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFileSaveAs()));
	tb->addAction(a);
	mn->addAction(a);
	
	a = new QAction(QIcon(add), tr("&Add questions..."), this);
	a->setShortcut(Qt::CTRL + Qt::Key_D);
	a->setToolTip(tr("Add questions from other package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotAddQuests()));
	tb->addAction(a);
	mn->addAction(a);
	
	a = new QAction(/*QIcon(add),*/ tr("A&dd questions TO package..."), this);
	//a->setShortcut(Qt::CTRL + Qt::Key_D);
	a->setToolTip(tr("Add questions to other package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotAddQuestsToPackage()));
	//tb->addAction(a);
	mn->addAction(a);
	
	QMenu *mnExport = mn->addMenu(QIcon(fileexport_xpm), tr("&Export in..."));
	connect(mnExport, SIGNAL(triggered(QAction*)), this, SLOT(slotExport(QAction*)));
	int loadingResult = centralWidgt->loadExportInfo(mnExport);
	mnExport->setEnabled(loadingResult);
	
	a = new QAction(QIcon(fileprint), tr("&Print"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_P);
	a->setToolTip(tr("Print the package"));
	a->setEnabled(loadingResult == 2);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotPrint()));
	//tb->addSeparator();
	//tb->addAction(a);
	mn->addSeparator();
	mn->addAction(a);
	
	a = new QAction(QIcon(exit_xpm), tr("E&xit"), this);
	a->setShortcut(Qt::ALT + Qt::Key_X);
	a->setToolTip(tr("Quit the editor"));
	connect(a, SIGNAL(triggered()), this, SLOT(close()));
	tb->addSeparator();
	tb->addAction(a);
	mn->addSeparator();
	mn->addAction(a);
	
	//edit menu&toolbar
	tb = new QToolBar(this);
	tb->setIconSize(QSize(16, 16));
	tb->setWindowTitle(tr("Edit Actions"));
	addToolBar(tb);
	mn = new QMenu(tr("&Edit"), this);
	menuBar()->addMenu(mn);	
	a = new QAction(QIcon(undo_xpm), tr("&Undo"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_Z);
	a->setToolTip(tr("Undo last changes"));
	tb->addAction(a);
	tb->addSeparator();
	mn->addAction(a);
	mn->addSeparator();
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotUndo()));
	a = new QAction(QIcon(editcut_xpm), tr("&Cut"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_X);
	a->setToolTip(tr("Cut selection to clipboard"));
	tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotCut()));
	a = new QAction(QIcon(editcopy_xpm), tr("C&opy"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_C);
	a->setToolTip(tr("Copy selection to clipboard"));
	tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotCopy()));
	a = new QAction(QIcon(editpaste_xpm), tr("&Paste"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_V);
	a->setToolTip(tr("Paste from clipboard"));
	tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotPaste()));
	mn->addSeparator();
	tb->addSeparator();
	
	//finds items
	a = new QAction(QIcon(find_xpm), tr("&Find in Package"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_F);
	a->setToolTip(tr("Finds text in package"));
	tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFindPackage()));
	
	a = new QAction(tr("F&ind in current quest"), this);
	//a->setShortcut(Qt::CTRL + Qt::Key_F);
	a->setToolTip(tr("Finds text in current quest"));
	//tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFindQuest()));
	
	a = new QAction(QIcon(next), tr("Fi&nd next"), this);
	a->setShortcut(Qt::Key_F3);
	a->setToolTip(tr("Finds next word"));
	tb->addAction(a);
	mn->addAction(a);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFindNext()));
	mn->addSeparator();
	
	//skip items
	QMenu *mns = mn->addMenu(tr("... words skiped"));
	connect(mns, SIGNAL(triggered(QAction*)), centralWidgt, SLOT(slotWordsSkiped(QAction*)));
	a = new QAction(tr("skiping"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_0);
	a->setData(QVariant(0));
	mns->addAction(a);
	mns->addSeparator();
	a = new QAction(tr("one"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_1);
	a->setData(QVariant(1));
	mns->addAction(a);
	a = new QAction(tr("two"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_2);
	a->setData(QVariant(2));
	mns->addAction(a);
	a = new QAction(tr("three"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_3);
	a->setData(QVariant(3));
	mns->addAction(a);
	a = new QAction(tr("four"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_4);
	a->setData(QVariant(4));
	mns->addAction(a);
	a = new QAction(tr("five"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_5);
	a->setData(QVariant(5));
	mns->addAction(a);
	mn->addSeparator();
	
	//capitalization items
	mns = mn->addMenu(tr("CAPITALIZATION in..."));
	connect(mns, SIGNAL(triggered(QAction*)), centralWidgt, SLOT(slotCapital(QAction*)));
	a = new QAction(tr("current quest"), this);
	a->setData(QVariant(0));
	mns->addAction(a);
	a = new QAction(tr("tournament"), this);
	a->setData(QVariant(1));
	mns->addAction(a);
	a = new QAction(QIcon(charset), tr("&Normalize"), this);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotNormalize()));
	a->setToolTip(tr("Normalize characters and paragraphs in text"));
	tb->addSeparator();
	tb->addAction(a);
	mn->addSeparator();
	mn->addAction(a);

	
	//jumps questions menu
	mn = new QMenu(tr("&Jumps"), this);
	menuBar()->addMenu(mn);
	a = new QAction(QIcon(i1rightarrow), tr("&Next question"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_E);
	a->setToolTip(tr("Jump to the next question"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotNextQuest()));
	mn->addAction(a);
	a = new QAction(QIcon(i1leftarrow), tr("&Previuos question"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_B);
	a->setToolTip(tr("Jump to the previous question"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotPrevQuest()));
	mn->addAction(a);
	a = new QAction(QIcon(i2rightarrow), tr("&Last question"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_End);
	a->setToolTip(tr("Jump to  the last question"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotLastQuest()));
	mn->addAction(a);
	a = new QAction(QIcon(i2leftarrow), tr("&First question"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_Home);
	a->setToolTip(tr("Jump to first question"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotFirstQuest()));
	mn->addAction(a);
	a = new QAction(tr("&Question N..."), this);
	a->setShortcut(Qt::CTRL + Qt::Key_G);
	a->setToolTip(tr("Jump to the specified question"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotQuestN()));
	mn->addAction(a);
	
	//package menu
	tb = new QToolBar(this);
	tb->setIconSize(QSize(16, 16));
	tb->setWindowTitle(tr("Package Actions"));
	addToolBar(tb);
	mn = new QMenu(tr("&Package"), this);
	menuBar()->addMenu(mn);
	a = new QAction(QIcon(wordprocessing_xpm), tr("&Options"), this);
	a->setShortcut(Qt::CTRL + Qt::Key_T);
	a->setToolTip(tr("Package's oprions"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotTournInfo()));
	tb->addAction(a);
	tb->addSeparator();
	mn->addAction(a);
	mn->addSeparator();
	a = new QAction(QIcon(sort_xpm), tr("&Sort"), this);
	a->setToolTip(tr("Sort questions in package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotSortQuests()));
	tb->addAction(a);
	mn->addAction(a);
	a = new QAction(QIcon(preview_xpm), tr("&Preview"), this);
	a->setToolTip(tr("Preview of package"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotPreview()));
	mn->addAction(a);
	if (loadingResult == 2)
		tb->addAction(a);
	else
	{
		a->setEnabled(false);
		//     , ..     
		//  -     
		//        
		emit signalNoPreview();
	} /* 		tb->addAction(a); */

	//tools menu
	mn = new QMenu(tr("&Tools"), this);
	connect(mn, SIGNAL(triggered(QAction*)), centralWidgt, SLOT(slotSpell(QAction*)));
	menuBar()->addMenu(mn);
	a = new QAction(tr("Spell &quets"), this);
	a->setData(QVariant(0));
	a->setEnabled(centralWidgt->isSpellingEnable());
	mn->addAction(a);
	a = new QAction(QIcon(spellcheck), tr("Spell &tournament"), this);
	a->setShortcut(Qt::Key_F7);
	a->setEnabled(centralWidgt->isSpellingEnable());
	a->setData(QVariant(1));
	mn->addAction(a);
	
	//options menu
	mn = new QMenu(tr("&Options"), this);
	menuBar()->addMenu(mn);
	a = new QAction(QIcon(configure_xpm), tr("&Options"), this);
	a->setToolTip(tr("Options"));
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotOptions()));
	mn->addAction(a);
	//help menu
	tb = new QToolBar(this);
	tb->setIconSize(QSize(16, 16));
	tb->setWindowTitle(tr("Help Actions"));
	addToolBar(tb);
	mn = new QMenu(tr("&Help"), this);
	menuBar()->addMenu(mn);
	a = new QAction(QIcon(help_xpm), tr("&Help"), this);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotHelp()));
	a->setToolTip(tr("Help"));
	tb->addAction(a);
	mn->addAction(a);
	tb->addSeparator();
	mn->addSeparator();
	a = new QAction(QIcon(idea), tr("&Tip of the day..."), this);
	connect(a, SIGNAL(triggered()), centralWidgt, SLOT(slotTOD()));
	a->setToolTip(tr("Show tip of the day"));
	//tb->addAction(a);
	mn->addAction(a);
	//tb->addSeparator();
	mn->addSeparator();
	a = new QAction(QIcon(quested_xpm), tr("&About"), this);
	connect(a, SIGNAL(triggered()), this, SLOT(slotAbout()));
	a->setToolTip(tr("About"));
	tb->addAction(a);
	mn->addAction(a);
} /* createMenu() */

/**
*     
*/
void
MainWin::slotCurQuestChanged(int aNumQuest, int aCountQuests)
{
	lbCurNum->setText(tr(" Current question: %1 ").arg(aNumQuest + 1));
	lbCountQuests->setText(tr(" Total questions: %1 ").arg(aCountQuests));
} /* MainWin::slotCurQuestChanged(int aNumQuest) */

/**
*    About
*/
void
MainWin::slotAbout()
{
	QE_AboutDlg(this).exec();
} /* MainWin::slotAbout() */

/**
*    
*/
void
MainWin::slotExport(QAction *ac)
{
	bool ok;
	int expNum = ac->data().toInt(&ok);
	if (ok)
		centralWidgt->exportTournament(expNum);
} /* MainWin::slotExport(QAction*) */

void MainWin::closeEvent( QCloseEvent *event )
{
	if ( centralWidgt->isPackageDirty() &&
			QMessageBox::information(this, tr("Attention"),
				tr("The package was changed.\nSave changes?"),
				QMessageBox::Yes | QMessageBox::Default, 
				QMessageBox::No | QMessageBox::Escape) == QMessageBox::Yes)
		centralWidgt->slotFileSave();
	centralWidgt->saveSettings();
	centralWidgt->releaseRes();
	event->accept();
} /* QE_CentralWidget::void closeEvent( QCloseEvent *e ) */

/**
* \brief     
*/
void
MainWin::restoreWinPosition()
{
	QSettings settings;
	settings.beginGroup("Window");
	bool extra = settings.value("IsMaximized", false).toBool();
	if (extra)
		showMaximized();
	else
	{
		//extra = settings.value("IsMinimized", false).toBool();
		//if (extra)
		//	showMinimized();
		//else
		//{
			QPoint p = settings.value("Position", QPoint()).toPoint();
			if (!p.isNull())
				move(p);
			QSize s = settings.value("Size", QSize()).toSize();
			if (s.isValid())
				resize(s);
		//} /* 		else */
	} /* 	else */
	settings.endGroup();
} /* MainWin::restoreWinPosition() */

/**
*     . 
*/
void
MainWin::slotNoPreview()
{
	QMessageBox::warning(this, tr("Templates loading"),
			tr("Can'nt find temaplate to html.\nPreview is not available."));
} /* MainWin::slotNoPreview() */

