#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QCheckBox>
#include <QLineEdit>
#include <QLabel>
#include <QPushButton>
#include <QString>
#include <QFileDialog>

#include "optwidget.h"

/**
* \brief .
* \params opt -   
*/
QE_OptWidget::QE_OptWidget(const QE_Options &opt, QWidget *parent):
	QWidget(parent)
{
	QVBoxLayout *topl = new QVBoxLayout(this);
	cbShowTip = new QCheckBox(tr("&Show tips at startup"));
	topl->addWidget(cbShowTip);
	//cbShowTip->setEnabled(false);
	cbShowTip->setCheckState(opt.showTip ? Qt::Checked : Qt::Unchecked);
	cbOpenLast = new QCheckBox(tr("&Open last package at startup"));
	topl->addWidget(cbOpenLast);
	cbOpenLast->setCheckState(opt.openLastTourn ? Qt::Checked : Qt::Unchecked);

	QHBoxLayout *hb = new QHBoxLayout;
	hb->setSpacing(2);
	QLabel *lb = new QLabel(tr("&Packages directory:"));
	hb->addWidget(lb);
	edTournDir = new QLineEdit(opt.tournDir, this);
	lb->setBuddy(edTournDir);
	hb->addWidget(edTournDir);
	QPushButton *pb = new QPushButton(tr("&Browse"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotBrowse()));
	hb->addWidget(pb);
	topl->addLayout(hb);
	
	cbJumpLast = new QCheckBox(tr("&Jump to last quest at"));
	topl->addWidget(cbJumpLast);
	cbJumpLast->setCheckState(opt.jumpLastQuest ? Qt::Checked : Qt::Unchecked);

	cbClearAuthEd = new QCheckBox(tr("&Clear author editor"));
	topl->addWidget(cbClearAuthEd);
	connect(cbClearAuthEd, SIGNAL(stateChanged(int)), this, SLOT(slotClearAuthChanged(int)));
	
	cbSetAuthEd = new QCheckBox(tr("&Set this author's name"));
	topl->addWidget(cbSetAuthEd);
	connect(cbSetAuthEd, SIGNAL(stateChanged(int)), this, SLOT(slotSetAuthChanged(int)));
	
	hb = new QHBoxLayout;
	topl->addLayout(hb);
	hb->setSpacing(2);
	lb = new QLabel(tr("&Editor name:"));
	hb->addWidget(lb);
	edAuthor = new QLineEdit(opt.author, this);
	hb->addWidget(edAuthor);
	lb->setBuddy(edAuthor);
	//     ,  
 	cbSetAuthEd->setCheckState(opt.setAuthorEd ? Qt::Checked : Qt::Unchecked);
	cbClearAuthEd->setCheckState(opt.clearAuthorEd ? Qt::Checked : Qt::Unchecked);
	
	cbNoSpelling = new QCheckBox(tr("&No spelling"));
	topl->addWidget(cbNoSpelling);
	cbNoSpelling->setCheckState(opt.noSpelling ? Qt::Checked : Qt::Unchecked);
	
	cbSaveAuthors = new QCheckBox(tr("&Save authors' list"));
	topl->addWidget(cbSaveAuthors);
	cbSaveAuthors->setCheckState(opt.saveAuthors ? Qt::Checked : Qt::Unchecked);
	
	cbPrintTurNewPage = new QCheckBox(tr("&Print every Tour on new page"));
	topl->addWidget(cbPrintTurNewPage);
	cbPrintTurNewPage->setCheckState(opt.printTurNewPage ? Qt::Checked : Qt::Unchecked);
	
	cbNormalize = new QCheckBox(tr("&Normalize text when saving quest"));
	topl->addWidget(cbNormalize);
	cbNormalize->setCheckState(opt.normalize ? Qt::Checked : Qt::Unchecked);

	setMaximumHeight(sizeHint().height());
} /* QE_OptWidget::QE_Options &opt, QWidget *parent = 0) */

/**
* \brief   
*/
void
QE_OptWidget::slotBrowse()
{
	QString dir = QFileDialog::getExistingDirectory(this, 
			tr("Choose a directory"));
	if (!dir.isEmpty())
		edTournDir->setText(dir);
} /* QE_OptWidget::slotBrowse() */

/**
* \brief    
* \params aPtrOpt -      
*/
void
QE_OptWidget::getOptions(QE_Options *aPtrOpt) const
{
	aPtrOpt->showTip = cbShowTip->checkState() == Qt::Checked;
	aPtrOpt->openLastTourn = cbOpenLast->checkState() == Qt::Checked;
	aPtrOpt->tournDir = edTournDir->text();
	aPtrOpt->jumpLastQuest = cbJumpLast->checkState() == Qt::Checked;
	aPtrOpt->author = edAuthor->text();
	aPtrOpt->clearAuthorEd = cbClearAuthEd->checkState() == Qt::Checked;
	aPtrOpt->setAuthorEd   = cbSetAuthEd->checkState() == Qt::Checked;
	aPtrOpt->noSpelling    = cbNoSpelling->checkState() == Qt::Checked;
	aPtrOpt->saveAuthors   = cbSaveAuthors->checkState() == Qt::Checked;
	aPtrOpt->printTurNewPage  = cbPrintTurNewPage->checkState() == Qt::Checked;
	aPtrOpt->normalize     = cbNormalize->checkState() == Qt::Checked;
} /* QE_OptWidget::getOprions(QE_Options *aPtrOpt) const */

/**
* \brief  /    
*/
void
QE_OptWidget::slotSetAuthChanged(int /* state */)
{
	edAuthor->setEnabled(cbSetAuthEd->isChecked());
} /* QE_OptWidget::slotSetAuthChanged(int state) */

/**
* \brief  /      
*/
void
QE_OptWidget::slotClearAuthChanged(int /* state */)
{
	cbSetAuthEd->setEnabled(cbClearAuthEd->isChecked());
	edAuthor->setEnabled(cbClearAuthEd->isChecked());
} /* QE_OptWidget::slotClearAuthChanged(int state) */
