#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QGroupBox>
#include <QFileDialog>
#include <QMainWindow>
#include <QStatusBar>
#include <QMessageBox>
#include <QMenu>
#include <QDir>
#include <QFileInfo>
#include <QStringList>
#include <QCoreApplication>
#include <QTextCharFormat>
#include <QTextEdit>
#include <QBrush>
#include <QSettings>
#include <QCloseEvent>

#if defined Q_WS_X11
#include <QAssistantClient>
#else
#include <QtAssistant\QAssistantClient>
#endif

#include <QPrinter>
#include <QPrintDialog>
#include <QInputDialog>

#include "qecentralwidget.h"
#include "qeedit.h"
#include "qeedit2.h"
#include "qelineedit.h"
#include "qelineedit2.h"
#include "listedit.h"
#include "listviewedit.h"
#include "tourninfodlg.h"
#include "quest.h"
#include "exporter.h"
#include "previewdlg.h"
#include "capdlg.h"
#include "sortdlg.h"
#include "optdlg.h"
#include "spelldlg.h"
#include "toddlg.h"
#include "choiceqdlg.h"
#include "finddlg.h"

#include "images/1rightarrow.xpm"
#include "images/1leftarrow.xpm"
#include "images/2rightarrow.xpm"
#include "images/2leftarrow.xpm"
#include "images/edittrash.xpm"

/**
* Constructor
* \param a_parent - 
*/
QE_CentralWidget::QE_CentralWidget(QMainWindow *a_parent):
	QWidget(a_parent),
#if defined Q_MS_WIN || defined QEDEBUG
	dataDir(QDir::currentPath()),
#else
	dataDir(PREFIX),
#endif
	findMode(F_NONE),
	htmlExporterNum(-1),
	aspellConfig(0),
	speller(0),
	assistant(0)
{
	parentWidget()->setWindowTitle("[*] - quested");
	
#if defined Q_WS_X11 && !defined QEDEBUG
	dataDir += "/share/quested";
#endif
	dataDir += QDir::separator();
	homeDataDir = QDir::homePath() + QDir::separator() + ".quested.data";
	QDir hdd(homeDataDir);
	if (!hdd.exists())
		QDir::home().mkdir(".quested.data");
	homeDataDir += QDir::separator();
	
	QCoreApplication::setOrganizationName("XBASoft");
	QCoreApplication::setApplicationName("Quested");
	
	readSettings();

	QVBoxLayout *topl = new QVBoxLayout(this);
	topl->setSpacing(0);
	topl->setMargin(2);
	lbInfo = new QLabel(tr("information"), this);
	QFont f(lbInfo->font());
	f.setBold(true);
	lbInfo->setFont(f);
	lbInfo->setAlignment(Qt::AlignCenter);
	lbInfo->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	topl->addWidget(lbInfo);
	edQuest = new QE_Edit(this, tr("&Question's editor"));
	connect(edQuest, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
	edAnswer = new QE_LineEdit2(this, tr(" A&nswer: "), tr(" S&core: "));
	connect(edAnswer, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
	edAuthor = new QE_ListViewEdit(this, tr(" A&uthor: "), tr("&Edit authors..."));
	connect(edAuthor, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
	if (options.saveAuthors)
		loadAuthorsList();
	edSource = new QE_ListEdit(this, tr(" &Source: "), tr("E&dit source..."));
	connect(edSource, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
	edAComm = new QE_Edit2(this, tr("&Author's comments"), tr("&Remark"));
	connect(edAComm, SIGNAL(textChanged()), this, SLOT(slotTextChanged()));
	//    
	for( int i = 0; i < CEDIT; ++i)
		switch(edOrder.at(i))
		{
			case 0:
				topl->addWidget(edQuest, 2);
				edits[i] = edQuest->editor();
				break;
			case 1:
				topl->addWidget(edAnswer);
				edits[i] = edAnswer->editor();
				//edits[++i] = edAnswer->editor2();
				break;
			case 2:
				topl->addWidget(edAuthor);
				edits[i] = edAuthor->editor();
				break;
			case 3:
				topl->addWidget(edSource);
				edits[i] = edSource->editor();
				break;
			case 4:
				topl->addWidget(edAComm, 1);
				edits[i] = edAComm->editor();
			default:
				break;
		} /* 		switch(edOrder.at(i)) */

	QGroupBox *gp = new QGroupBox(tr("Questions' &operations"), this);
	topl->addWidget(gp);
	QHBoxLayout *hl = new QHBoxLayout(gp);
	hl->setSpacing(1);
	hl->setMargin(1);
	pbNext = new QPushButton(QIcon(i1rightarrow), tr("N&ext"), gp);
	connect(pbNext, SIGNAL(clicked(bool)), this, SLOT(slotNextQuest()));
	hl->addWidget(pbNext);
	pbPrev = new QPushButton(QIcon(i1leftarrow), tr("&Previous"), gp);
	connect(pbPrev, SIGNAL(clicked(bool)), this, SLOT(slotPrevQuest()));
	hl->addWidget(pbPrev);
	pbLast = new QPushButton(QIcon(i2rightarrow), tr("&Last"), gp);
	connect(pbLast, SIGNAL(clicked(bool)), this, SLOT(slotLastQuest()));
	hl->addWidget(pbLast);
	pbFirst = new QPushButton(QIcon(i2leftarrow), tr("&First"), gp);
	connect(pbFirst, SIGNAL(clicked(bool)), this, SLOT(slotFirstQuest()));
	hl->addWidget(pbFirst);
	pbDelete = new QPushButton(QIcon(edittrash), tr("&Delete"), gp);
	connect(pbDelete, SIGNAL(clicked(bool)), this, SLOT(slotDelCurQuest()));
	hl->addWidget(pbDelete);
	hl->addStretch(1);

	stbStatusBar = a_parent->statusBar();

	if (!options.noSpelling)
		createSpeller();

	QCoreApplication *qApp = QCoreApplication::instance();
	connect(this, SIGNAL(noStartupFile(const QString&)), 
			this, SLOT(slotNoStartupFile(const QString&)), Qt::QueuedConnection);
	bool argFile = false;	//     
	if (qApp->argc() > 1)
	{
		fileName = qApp->argv()[1];
		argFile = true;
	} /* 	if (qApp->argc() > 1) */
	if ( (argFile  || options.openLastTourn) && !fileName.isEmpty())
		if (!QFile::exists(fileName))
		{
			emit noStartupFile(fileName);
			slotFileNew();
		} /* 		if (!QFile::exists(fileName)) */
		else
			openFile();
	else
		slotFileNew();

	loadCapitalWord();

	if (options.showTip)
	{
		connect(this, SIGNAL(showTOD()), this, SLOT(slotTOD()), Qt::QueuedConnection);
		emit showTOD();
	} /* 	if (options.showTip) */	
} /* QE_CentralWidget::*a_parent) */

/**
*    
*/
void
QE_CentralWidget::slotFileNew()
{
	lbInfo->setText(tr("You can set package options into menu"));
	curNumQuest = 0;
	packageDirty = false;
	fileName.clear();
	mTournament.clear();
	showCurrentQuest();
	parentWidget()->setWindowModified(false);
} /* QE_CentralWidget::newPackage() */

/**
*       
*/
void
QE_CentralWidget::slotTournInfo()
{
	QE_TournInfoDialog dlg(mTournament.ptrTournInfo(), parentWidget());
	if(dlg.exec() == QDialog::Accepted)
	{
		QE_TournInfoData ti;
		mTournament.getTournamentInfo(&ti);
		lbInfo->setText(ti.title);
		packageDirty = true;
	} /* 	if(dlg.exec() == QDialog::Accepted) */
} /* QE_CentralWidget::slotTournInfo() */

/**
*     
*/
void
QE_CentralWidget::slotFileSave()
{
	if (!fileName.isEmpty())
	{
		saveCurQuest(true);
		mTournament.save(fileName);
		packageDirty = false;
		parentWidget()->setWindowModified(false);
	} /* 	if (!fileName.isEmpty()) */
	else
		slotFileSaveAs();
} /* QE_CentralWidget::slotFileSave() */

/**
*  [  . . [  
*   
*/
void
QE_CentralWidget::slotNextQuest()
{
	setQuest(curNumQuest + 1);
} /* QE_CentralWidget::slotNextQuest() */

/**
*   
*/
void
QE_CentralWidget::slotFileOpen()
{
	QString fn = QFileDialog::getOpenFileName(this, tr("Open quested File"),
                 options.tournDir.isEmpty() ? QDir::currentPath() : options.tournDir,
				 tr("Package's files (*.qed)"));
    if (fn.isEmpty())
        return;
	fileName = fn;
	openFile();
} /* QE_CentralWidget::slotFileOpen() */

/**
*       
*/
void
QE_CentralWidget::showCurrentQuest()
{
	if (curNumQuest >= mTournament.countQuests())
		mTournament.addPtrQuest(new QE_Quest);
	
	emit curQuestChanged(curNumQuest, mTournament.countQuests());

	pbPrev->setEnabled(curNumQuest);
	pbFirst->setEnabled(curNumQuest);
	pbLast->setEnabled(curNumQuest + 1 < mTournament.countQuests());
	pbDelete->setEnabled(mTournament.countQuests() > 1);
	
	QE_Quest *questPtr = mTournament.ptrQuest(curNumQuest);
	edQuest->setText(questPtr->text());
	edAnswer->setText(questPtr->answer());
	edAnswer->setText2(questPtr->score());
	if ( questPtr->countAuthors())
		edAuthor->setList(questPtr->authors());
	else
		if (options.clearAuthorEd)
			edAuthor->setText(options.setAuthorEd ? options.author : QString());
	edAComm->setText(questPtr->comment());
	edAComm->setText2(questPtr->remarks());
	edSource->setList(questPtr->sources());
	isDirty = false;
} /* QE_CentralWidget::showCurrentQuest() */

/**
*  [  . . [  
*   
*/
void
QE_CentralWidget::slotPrevQuest()
{
	setQuest(curNumQuest - 1);
} /* QE_CentralWidget::slotPrevQuest() */

/**
*  [   . [  
*   
*/
void
QE_CentralWidget::slotFirstQuest()
{
	setQuest(0);
} /* QE_CentralWidget::slotFirstQuest() */

/**
*  [   . [  
*   
*/
void
QE_CentralWidget::slotLastQuest()
{
	setQuest(mTournament.countQuests() ? mTournament.countQuests() - 1 : 0);
} /* QE_CentralWidget::slotLastQuest() */

/**
*      
*/
void
QE_CentralWidget::slotFileSaveAs()
{
	QFileDialog fileDlg(this, tr("Select file for saving"), 
				options.tournDir.isEmpty() ? QDir::currentPath() : options.tournDir, 
				tr("Package's files (*.qed)"));
	fileDlg.setDefaultSuffix("qed");
	fileDlg.exec();
	QStringList files(fileDlg.selectedFiles());
	if(!files.isEmpty())
	{
		fileName = files[0];
		saveCurQuest(true);
		mTournament.save(fileName);
		packageDirty = false;
		parentWidget()->setWindowTitle(fileName + QString("[*] - quested"));
		parentWidget()->setWindowModified(false);
	} /* 	if(!fn.isEmpty()) */
} /* QE_CentralWidget::slotFileSaveAs() */

/**
*    
*/
void
QE_CentralWidget::slotTextChanged()
{
	parentWidget()->setWindowModified(true);
	isDirty = true;
} /* QE_CentralWidget::slotTextChanged() */

void
QE_CentralWidget::saveCurQuest(bool forceSave)
{
	if (isDirty && (forceSave || QMessageBox::question(this, tr("Question saving"), 
				tr("Quest changed.\nSave changes?"), 
				QMessageBox::Yes | QMessageBox::Default, 
				QMessageBox::No | QMessageBox::Escape, 
				QMessageBox::NoButton) == QMessageBox::Yes))
	{
		//QString t(edAnswer->text());
		//qDebug("%x %x %x %x %x %x", t.at(0).unicode(), t.at(1).unicode(), t.at(2).unicode(),
		//		t.at(3).unicode(), t.at(4).unicode(), t.at(5).unicode());
		if (options.normalize)
			slotNormalize();

		mTournament.ptrQuest(curNumQuest)->setQuestInfo(edQuest->text(),
		   	edAnswer->text(), edAuthor->list(),
			edAComm->text(), edAnswer->text2(), edSource->list(),
			edAComm->text2());
		//     
		edAuthor->addVariant(edAuthor->text());
		
		isDirty = false;
		packageDirty = true;
	} /* 	::Yes)) */
} /* QE_CentralWidget::saveCurQuest() */

int
QE_CentralWidget::loadExportInfo(QMenu *a_Menu)
{
	QString templDirName(dataDir + "templates/");
	QDir templDir(templDirName);
	if (!templDir.exists())
		return 0;

	QAction *ac;
	QFileInfoList list = templDir.entryInfoList(QStringList("*.tpl"), QDir::Files);
	int j = 0;
	for(int i = 0; i < list.size(); ++i)
	{
		QFileInfo fi = list.at(i);
		ac = loadOneTemplate(templDirName + fi.fileName());
		if (ac)
		{
			if (ac->text() == "html")
				htmlExporterNum = j;
			ac->setData(j++);
			a_Menu->addAction(ac);
		} /* 		if (ac) */
	} /* 	for(int i = 0; i < list.size(); ++i) */
	return htmlExporterNum >= 0 ? 2 : (int)j > 0;
} /* QE_CentralWidget::loadExportInfo(QMenu *a_Menu) */

/**
*     
* \param a_fileName -   
* \returns  QAction    0  
*/
QAction* QE_CentralWidget::loadOneTemplate(const QString &a_fileName)
{
	QE_Exporter *expr = new QE_Exporter;
	if (expr->load(a_fileName))
	{
		exporter.append(expr);
		QE_Exporter *lastExp = exporter.last();
		return new QAction(lastExp->shortDescr(), parentWidget());
	} /* 	if (expr->load(a_fileName)) */
	else
	{
		delete expr;
		return 0;
	} /* 	else */
} /* QAction* loadOneTemplate(const QString &a_fileName) */

void
QE_CentralWidget::exportTournament(int aNumTemplate)
{
	QFileDialog fileDlg(this, tr("Choose file for export"), 
				options.tournDir.isEmpty() ? QDir::currentPath() : options.tournDir, 
				exporter.at(aNumTemplate)->fileType() + " (*." + 
				exporter.at(aNumTemplate)->fileExt() + ")");
	fileDlg.setDefaultSuffix(exporter.at(aNumTemplate)->fileExt());
	fileDlg.exec();
	fileDlg.setLabelText(QFileDialog::Accept, tr("&Save"));
	fileDlg.setAcceptMode(QFileDialog::AcceptSave);
	QStringList files(fileDlg.selectedFiles());
	if (!files.isEmpty())
	{
		QFile file(files[0]);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
			QMessageBox::critical(this, tr("Writing file"),
					tr("Cann't open %1 for writing").arg(files[0]));
		else
		{
			QByteArray resultStr;
			exporter.at(aNumTemplate)->exportToString(mTournament, &resultStr);
			file.write(resultStr);
			file.close();
		} /* 		else */
	} /* 	if (!fileName.isEmpty()) */
} /* QE_CentralWidget::slotExport(QAction *ac) */

/**
*    (fileName)
*/
void
QE_CentralWidget::openFile()
{
	stbStatusBar->showMessage(tr("Opening package"));
	mTournament.clear();
	if (mTournament.load(fileName))
	{
		lbInfo->setText(mTournament.ptrTournInfo()->title);
		curNumQuest = 0;
		showCurrentQuest();
		if (options.jumpLastQuest)
			slotLastQuest();
		packageDirty = false;
		edAuthor->addVariants(mTournament.authors());
		parentWidget()->setWindowTitle(fileName + QString("[*]- quested"));
		parentWidget()->setWindowModified(false);
	} /* 	if (mTournament.load(fileName)) */
	else
			QMessageBox::warning(this, tr("Error opening file"), tr("Can\'t open %1").arg(fileName));
	stbStatusBar->clearMessage();
} /* QE_CentralWidget::openFile() */

/**
*   
*/
void
QE_CentralWidget::slotPreview()
{
	QString resultStr;
	exporter.at(htmlExporterNum)->exportToUnicodeString(mTournament, &resultStr);
	QE_PreviewDlg dlg(resultStr, this);
	dlg.exec();
} /* QE_CentralWidget::slotPreview() */

/**
*   "... "
*/
void
QE_CentralWidget::slotWordsSkiped(QAction *ac)
{
	bool ok;
	int count;
	QString words(" [");
	count = ac->data().toInt(&ok);
	if (ok)
	{
		switch (count)
		{
			case 0:
				edQuest->insertText(tr("[skiping]"));
				return;
			case 1:
				words += tr("one word");
				break;
			case 2:
			case 3:
			case 4:
				words += tr("%1 words").arg(count);
				break;
			case 5:
				words += tr("five words");
			default:
				break;
		} /* 		switch (count) */
		words += tr(" skiped] ");
		edQuest->insertText(words);
	} /* 	if (ok) */
} /* QE_CentralWidget::slotWordsSkiped(QAction *ac) */

QE_CentralWidget::~QE_CentralWidget()
{
} /* ~QE_CentralWidget() */

/** 
*       
*/
void
QE_CentralWidget::slotAddQuests()
{
	saveCurQuest();
	QString fn = QFileDialog::getOpenFileName(this, tr("Open quested File"),
                        QDir::currentPath(), tr("Package's files (*.qed)"));
    if (fn.isEmpty())
        return;
	QE_Tournament tourn1;
	tourn1.load(fn);
	int i, cq = tourn1.countQuests();
	for (i = 0; i < cq; ++i)
		mTournament.addPtrQuest(new QE_Quest(*(tourn1.ptrQuest(i))));
	packageDirty = true;
	emit curQuestChanged(curNumQuest, mTournament.countQuests());
} /* QE_CentralWidget::slotAddQuests() */


/**
*   
*/
void
QE_CentralWidget::slotCapital(QAction *ac)
{
	bool ok;
	int tempNumCurQuest = curNumQuest;
	int type = ac->data().toInt(&ok);
	QE_CapitalDlg dlg(this, tr("CAPITALIZATION"), capitalWords, 
			edits, 1, Qt::blue, type ? mTournament.countQuests() : 0);
	if (dlg.isTransformNeeded())
		isDirty = dlg.exec() == QDialog::Accepted;
	if (type)
	{
		curNumQuest = tempNumCurQuest;
		showCurrentQuest();
	} /* 		if (type) */
} /* QE_CentralWidget::slotCapital(QAction *ac) */

/**
* \brief   [ 
*/
void
QE_CentralWidget::loadCapitalWord()
{
	QFile file(dataDir + "capitalwords.txt");
	if ( !file.open(QIODevice::ReadOnly | QIODevice::Text) )
		return;
	
	capitalWords.clear();
	QTextStream in(&file);
	in.setCodec("UTF-8");
	while ( !in.atEnd() )
		capitalWords << in.readLine();
} /* QE_CentralWidget::loadCapitalWord() */

/**
*     
*/
void
QE_CentralWidget::slotSortQuests()
{
	if ( QE_SortDlg(this, &mTournament).exec() == QDialog::Accepted )
	{
		showCurrentQuest();
		packageDirty = true;
		parentWidget()->setWindowModified(true);
	} /* 	if ( QE_SortDlg(this, &mTournament).exec() == QDialog::Accepted ) */
} /* QE_CentralWidget::slotSortQuests() */

/**
*    
*/
void
QE_CentralWidget::slotOptions()
{
	if ( QE_OptDlg(this, &options, &edOrder).exec() == QDialog::Accepted )
		saveSettings();
} /* QE_CentralWidget::slotOptions() */

/**
* \brief [ 
*/
void
QE_CentralWidget::saveSettings()
{
	QSettings settings;
	settings.beginGroup("Tournament");
	settings.setValue("OpenLastTournament", options.openLastTourn);
	settings.setValue("ShowTips", options.showTip);
	settings.setValue("PackageDir", options.tournDir);
	settings.setValue("JumpToLastQuest", options.jumpLastQuest);
	settings.setValue("Author", options.author);
	settings.setValue("ClearAuthorEditor", options.clearAuthorEd);
	settings.setValue("SetAuthorEditor", options.setAuthorEd);
	settings.setValue("NoSpelling", options.noSpelling);
	settings.setValue("SaveAuthors", options.saveAuthors);
	settings.setValue("PrintTurNewPage", options.printTurNewPage);
	settings.setValue("Normalize", options.normalize);
	
	settings.setValue("LastPackage", fileName);
	//[  
	//   QVariant  [  [
	QVariantList varlist;
	for(int i = 0; i < CEDIT; ++i)
		varlist << edOrder.at(i);
	settings.setValue("EditorOrder", varlist);
	settings.endGroup();
	settings.beginGroup("Window");
	settings.setValue("Size", parentWidget()->size());
	settings.setValue("IsMaximized", parentWidget()->isMaximized());
	settings.setValue("IsMinimized", parentWidget()->isMinimized());
	settings.setValue("Position", parentWidget()->pos());
	settings.endGroup();

	if (options.saveAuthors)
		saveAuthorsList();
} /* QE_CentralWidget::saveSettings() */

/**
* \brief   
*/
void
QE_CentralWidget::readSettings()
{
	QSettings settings;
	settings.beginGroup("Tournament");
	options.openLastTourn = settings.value("OpenLastTournament", false).toBool();
	options.showTip       = settings.value("ShowTips", false).toBool();
	options.tournDir      = settings.value("PackageDir", QString()).toString();
	options.jumpLastQuest = settings.value("JumpToLastQuest", false).toBool();
	options.author        = settings.value("Author", QString()).toString();
	options.clearAuthorEd = settings.value("ClearAuthorEditor", false).toBool();
	options.setAuthorEd   = settings.value("SetAuthorEditor", false).toBool();
	options.noSpelling    = settings.value("NoSpelling", false).toBool();
	options.saveAuthors   = settings.value("SaveAuthors", false).toBool();
	options.printTurNewPage  = settings.value("PrintTurNewPage", false).toBool();
	options.normalize     = settings.value("Normalize", false).toBool();
	fileName              = settings.value("LastPackage", QString()).toString();
	QVariantList varlist;
	int i;
	for(i = 0; i < CEDIT; ++i)
		varlist << i;
	varlist = settings.value("EditorOrder", varlist).toList();
	for(i = 0; i < CEDIT; ++i)
		edOrder << varlist.at(i).toInt();
	settings.endGroup();
} /* QE_CentralWidget::readSettings() */

/**
* \brief  
*/
void
QE_CentralWidget::createSpeller()
{
	aspellConfig = new_aspell_config();

#ifdef Q_WS_WIN
	// windows    Aspell
	QSettings key("HKEY_LOCAL_MACHINE\\Software\\Aspell", QSettings::NativeFormat);
	
	QByteArray path(key.value("").toByteArray());
	QByteArray dictDir(path);
	dictDir += "\\dict";
	path += "\\data";
	aspell_config_replace(aspellConfig, "dict-dir", dictDir.constData());
	aspell_config_replace(aspellConfig, "data-dir", path.constData());
#endif
	aspell_config_replace(aspellConfig, "lang", "ru");
	aspell_config_replace(aspellConfig, "encoding", "koi8-r");

	AspellCanHaveError *possible_err = new_aspell_speller(aspellConfig); 

	if (aspell_error_number(possible_err) == 0) 
		speller = to_aspell_speller(possible_err); 
	else
		QMessageBox::warning(this, tr("Speller init"),
				tr("Speller init error %1.\n"
					"Spelling not available.").arg(aspell_error_message(possible_err)));
} /* QE_CentralWidget::createSpeller() */

/**
* \brief    (    ..)
*/
void
QE_CentralWidget::releaseRes()
{
	if (speller)
		delete_aspell_speller(speller);
	if (aspellConfig)
		delete_aspell_config(aspellConfig);
} /* QE_CentralWidget::releaseRes() */

/**
*    
*/
void
QE_CentralWidget::slotSpell(QAction *ac)
{
	QE_SpellDlg dlg(this, tr("Spell checking"), speller, 
			edits, CEDIT, Qt::red, ac->data().toInt() ? mTournament.countQuests() : 0);
	isDirty = dlg.exec() == QDialog::Accepted;
} /* QE_CentralWidget::slotSpell(QAction *ac) */

/**
*   
*/
void
QE_CentralWidget::slotHelp()
{
	if (!assistant)
	{
		QStringList cmdLst("-profile");
		cmdLst << dataDir + QString("help") + QDir::separator()	+ QString("quested.adp");
		assistant = new QAssistantClient("");
		assistant->setArguments(cmdLst);
	} /* 	if (!assistant) */	
	assistant->showPage(dataDir + QString("help") + QDir::separator() + QString("intro.htm"));
} /* QE_CentralWidget::slotHelp() */

/**
* slot undo
*/
void
QE_CentralWidget::slotUndo()
{
	QTextEdit *ed = dynamic_cast<QTextEdit*>(focusWidget());
	if (ed)
		ed->document()->undo();
} /* QE_CentralWidget::slotUndo() */

/**
* slot Cut
*/
void
QE_CentralWidget::slotCut()
{
	QTextEdit *ed = dynamic_cast<QTextEdit*>(focusWidget());
	if (ed)
		ed->cut();
} /* QE_CentralWidget::slotCut() */

/**
* slot copy
*/
void
QE_CentralWidget::slotCopy()
{
	QTextEdit *ed = dynamic_cast<QTextEdit*>(focusWidget());
	if (ed)
		ed->copy();
} /* QE_CentralWidget::slotCopy() */

/**
* slot paste
*/
void
QE_CentralWidget::slotPaste()
{
	QTextEdit *ed = dynamic_cast<QTextEdit*>(focusWidget());
	if (ed)
		ed->paste();
} /* QE_CentralWidget::slotPaste() */

/**
* \brief    
*/
void
QE_CentralWidget::loadAuthorsList()
{
	QFile file(homeDataDir + "authors.lst");
	if (file.open(QFile::ReadOnly))
	{
		QTextStream strm(&file);
		strm.setCodec("UTF-8");
		strm.setAutoDetectUnicode(true);
		
		QSet<QString> authors;

		while (!strm.atEnd())
			authors.insert(strm.readLine());

		edAuthor->addVariants(authors);
	} /* 	if (file.open(QFile::ReadOnly)) */
} /* QE_CentralWidget::loadAuthorsList() */

/**
* \brief []   
*/
void
QE_CentralWidget::saveAuthorsList()
{
	QSet<QString> authors(edAuthor->variants());
	QFile file(homeDataDir + "authors.lst");
	if (file.open(QFile::WriteOnly))
	{
		QTextStream strm(&file);
		strm.setCodec("UTF-8");
		//strm.setAutoDetectUnicode(true);
		foreach(QString value, authors)
			strm << value << '\n';
	} /* 	if (file.open(QFile::ReadOnly)) */
} /* QE_CentralWidget::saveAuthorsList() */


/**
* slot Print
*/
void
QE_CentralWidget::slotPrint()
{
#ifndef QT_NO_PRINTER
    QPrinter printer(QPrinter::HighResolution);
    printer.setFullPage(true);
	//printer.setPrintProgram("/usr/bin/kghostview");

    QPrintDialog dlg(&printer, this);
    if (dlg.exec() == QDialog::Accepted)
   	{
		QString resultStr;
		QTextDocument document(this);
		if (!options.printTurNewPage || !mTournament.ptrTournInfo()->cTurQuests)	//  
		{
			exporter.at(htmlExporterNum)->exportToUnicodeString(mTournament, &resultStr);
			document.setHtml(resultStr);
			document.print(&printer);
		} /* 		if (!options.printTurNewPage && !tournament.ptrTournInfo()->cTurQuests)	//   */
		else	//  
		{
			int countQuests = mTournament.countQuests();
			int countTurQuests = mTournament.ptrTournInfo()->cTurQuests;
			int countTurs = countQuests / countTurQuests;
			int i;
			exporter.at(htmlExporterNum)->exportToUnicodeString(mTournament, &resultStr, 0);
			document.setHtml(resultStr);
			document.print(&printer);
			for (i = 1; i < countTurs; ++i)
			{
				exporter.at(htmlExporterNum)->exportToUnicodeString(mTournament, &resultStr, i, false);
				document.setHtml(resultStr);
				document.print(&printer);
			} /* 			for (i = 1; i < countTurs; ++i) */
		} /* 		else */
	} /*     if (dlg.exec() == QDialog::Accepted) */ 
#endif
} /* QE_CentralWidget::slotPrint() */

/**
* \brief    " "
*/
void
QE_CentralWidget::slotTOD()
{
	QE_TODDlg(this, dataDir + "quested.tips", &(options.showTip)).exec();
} /* MainWin::slotTOD() */

/**
* \brief slotSetPar() -   
*
*          
*/
void
QE_CentralWidget::setPars(QTextEdit *aEditor)
{
	QString par("<p");
	QChar   rb('>'), sp(' ');
	int parInd = 0;
	bool isFirstPar = true;
	int spcount;
	
	QString qHtml(aEditor->toHtml());
	//[   
	while ((parInd = qHtml.indexOf(par, parInd)) != -1)
	{
		//      
		parInd = qHtml.indexOf(rb, parInd) + 1;
		if (isFirstPar)
		{
			isFirstPar = false;
			continue;
		} /* 		if (isFirstPar) */
		//     
		for(spcount = 3; qHtml[parInd] == sp && spcount > 0; --spcount, ++parInd)
			;
		if (spcount)
			qHtml.insert(parInd, QString(spcount, sp));
	} /* 	while ((parInd = qHtml.indexOf(par, parInd)) != -1) */
	aEditor->setHtml(qHtml);
} /* QE_CentralWidget::slotSetPar() */

/**
*  [    . [  
*   
*/
void
QE_CentralWidget::slotQuestN()
{
	bool ok;
    int i = QInputDialog::getInteger(this, tr("Jump to quest"),
                                         tr("Enter number of quest:"), 1, 1,
										 mTournament.countQuests(), 1, &ok);
    if (ok)
	{
		setQuest(i - 1);
	} /*     if (ok) */
} /* QE_CentralWidget::slotQuestN() */

/**
*    
*/
void
QE_CentralWidget::slotDelCurQuest()
{
	mTournament.removeQuest(curNumQuest);
	if (curNumQuest >= mTournament.countQuests())
		--curNumQuest;
	showCurrentQuest();
} /* QE_CentralWidget::slotDelCurQuest() */

/**
* \brief     
* \param aQuestNum -   
* \warning      
*/
void
QE_CentralWidget::setQuest(int aQuestNum)
{
	if (edQuest->text().isEmpty() || edAnswer->text().isEmpty() ||
		   	edSource->text().isEmpty())
		QMessageBox::warning(this, tr("Jump to next question"),
			tr("You should specify atleast question's text, answer and source"));
	else
	{
		saveCurQuest();
		curNumQuest = aQuestNum;
		showCurrentQuest();
	} /* 	else */
} /* QE_CentralWidget::setQuest(int aQuestNum) */

/**
* \brief     
*
*      [    
*/
void
QE_CentralWidget::slotAddQuestsToPackage()
{
	QString fn = QFileDialog::getOpenFileName(this, tr("Open quested File"),
                 options.tournDir.isEmpty() ? QDir::currentPath() : options.tournDir,
				 tr("Package's files (*.qed)"));
    if (!fn.isEmpty())
	{
		QE_ChoiceQuestDlg dlg(this, mTournament, true);
		if (dlg.exec() == QDialog::Accepted)
		{
			QE_Tournament destTourn;
			if (!destTourn.load(fn))
				QMessageBox::warning(this, tr("Loading error"),
					tr("Can't load package"));
			else
			{	
				QVector<int> quests(dlg.selectedQuests());
				foreach(int i, quests)
					destTourn.addPtrQuest(new QE_Quest(*(mTournament.ptrQuest(i))));
				destTourn.save(fn);
			} /* 			else */
		} /* 	if (dlg.exec() == QDialog::Accepted) */
	} /*     if (!fn.isEmpty()) */
} /* QE_CentralWidget::slotAddQuestsToPackage() */

/**
* \brief        
*/
void
QE_CentralWidget::slotNoStartupFile(const QString &fn)
{
	QMessageBox::warning(this, tr("Error opening file"), tr("File %1 does not exists.").arg(fn));
} /* QE_CentralWidget::slotNoStartupFile() */

/**
* \brief     
*/
void
QE_CentralWidget::slotFindQuest()
{
	findMode = F_QUEST;
	curFindQuest = curNumQuest;
	find(true);
} /* QE_CentralWidget::slotFindQuest() */

/**
* \brief     
*/
void
QE_CentralWidget::slotFindPackage()
{
	findMode = F_PACK;
	curFindQuest = 0;
	find(true);
} /* QE_CentralWidget::slotFindPackage() */

/**
* \brief   
*/
void
QE_CentralWidget::slotFindNext()
{
	if (findMode == F_NONE)
		slotFindPackage();
	else
		find();
} /* QE_CentralWidget::slotFindNext() */

/**
* \brief   
*
*     .   , 
*      .
* \param isStart -  true,    
*/
void
QE_CentralWidget::find(bool isStart)
{
	if (isStart)
	{	//   ,   
		QE_FindDlg dlg(this);
		if (dlg.exec() == QDialog::Accepted)
		{
			textToFind = dlg.text();
			isCaseSensitive = dlg.isCaseSensitive();
			findPos = 0;
			curFEdit = F_EQUEST;	//    */
			curFEditor = edQuest->editor();
			curFText = mTournament.ptrQuest(curFindQuest)->text();
		} /* 		if (dlg.exec() == QDialog::Accepted) */
		else
		{
			findMode = F_NONE;
			return;
		} /* 		else */
	} /* 	if (isStart) */
	//   
	while (1)
	{
		if (curFindQuest < mTournament.countQuests())
			findPos = curFText.indexOf(textToFind, findPos,
					isCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive);
		if (findPos != -1)
		{	//   
			// [   
			if (curFindQuest != curNumQuest)
			{
				curNumQuest = curFindQuest;
				setQuest(curFindQuest);
			} /* 			if (curFindQuest != curNumQuest) */
			// 
			//   -     
			if (F_ESRC == curFEdit)
				curFEditor->setPlainText(curFText);
			if (isCaseSensitive)
				curFEditor->find(textToFind, QTextDocument::FindCaseSensitively);
			else
				curFEditor->find(textToFind);
			//[  .   
			findPos += textToFind.length();
			break;
		} /* 		if (findPos != -1) */
		else
		{
			//      - [  .
			//  
			switch (curFEdit)
			{
				case F_EQUEST:
					curFEdit = F_EANSWER;
					curFEditor = edAnswer->editor();
					curFText = mTournament.ptrQuest(curFindQuest)->answer();
					break;
				case F_EANSWER:
					curFEdit = F_ESCORE;
					curFEditor = edAnswer->editor2();
					curFText = mTournament.ptrQuest(curFindQuest)->score();
					break;
				case F_ESCORE:
					curFEdit = F_EAUTHOR;
					curFEditor = edAuthor->editor();
					curFText = mTournament.ptrQuest(curFindQuest)->author();
					break;
				case F_EAUTHOR:
					curFEdit = F_ECOMM;
					curFEditor = edAComm->editor();
					curFText = mTournament.ptrQuest(curFindQuest)->comment();
					break;
				case F_ECOMM:
					curFEdit = F_EREM;
					curFEditor = edAComm->editor2();
					curFText = mTournament.ptrQuest(curFindQuest)->remarks();
					break;
				case F_EREM:
					curFEdit = F_ESRC;
					numSrc = 0; //   
					curFEditor = edSource->editor();
					curFText = mTournament.ptrQuest(curFindQuest)->sources().at(numSrc);
					break;
				case F_ESRC:
					++numSrc;
					if (numSrc < mTournament.ptrQuest(curFindQuest)->sources().count())
						curFText = mTournament.ptrQuest(curFindQuest)->sources().at(numSrc);
					else
					{
						curFEdit = F_EQUEST;
						curFEditor = edQuest->editor();
						++curFindQuest;
					} /* 					else */
					break;
				default:
					qDebug("wrong caseee");
			} /* 			switch (curFEdit) */
			
			findPos = 0;
			if (curFEdit != F_EQUEST)	//      
				continue;				//,   
			// ,    
			if (findMode == F_QUEST || curFindQuest >= mTournament.countQuests())
			{
				QMessageBox::information(this, tr("Finding"), tr("Search is end"));
				break;
			} /* 			if (findMode = F_QUEST || curFindQuest >= mTournament.countQuests()) */
		} /* 		else */
	} /* 	while (1) */
} /* QE_CentralWidget::find(bool isStart = false) */

/**
* \brief        
*
*  :   "-"  <<  >>, 
* ""  ..., " "  
* \param aEditor -    
*/
void
QE_CentralWidget::normalizeChars(QTextEdit *aEditor)
{
	enum { LEFTQUOTES=0xAB, RIGHTQUOTES = 0xBB, POINTS = 0x2026, NUMBER = 0x2116, 
		LONGDEFIS = 0x2013, LONGLONGDEFIS= 0x2014};
	static struct 
	{
		unsigned unic;
		QString  replText;
	} replPairs[] = 
	{
		{ LEFTQUOTES, QString("\"")}, 
		{ RIGHTQUOTES, QString("\"")}, 
		{ POINTS, QString("...")}, 
		{ NUMBER, QString("N")}, 
		{ LONGDEFIS, QString("-")}, 
		{ LONGLONGDEFIS, QString("-")}, 
	};
	
	QTextCursor tc = aEditor->textCursor();
	tc.movePosition(QTextCursor::Start);
	while(!tc.atEnd())
	{
		tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor);
		unsigned unic = tc.selectedText().at(0).unicode();
		for(unsigned i = 0; i < sizeof(replPairs)/sizeof(replPairs[0]); ++i)
			if(unic == replPairs[i].unic)
			{
				tc.insertText(replPairs[i].replText);
				break;
			} /* if(unic == replPairs[i].unic) */
		tc.setPosition(tc.position());
	} /* 		while(!tc.atEnd()) */
} /* QE_CentralWidget::normalize(QTextEdit *aEditor) */


/**
* \brief   
*
*          [ 
*      
*/
void
QE_CentralWidget::slotNormalize()
{
	normalizeChars(edQuest->editor());
	normalizeChars(edAnswer->editor());
	normalizeChars(edAnswer->editor2());
	normalizeChars(edAComm->editor());
	normalizeChars(edAComm->editor2());
	QStringList list = edSource->list();
	//foreach(QString src, list)
	int i;
	for(i = 0; i < list.size(); ++i)
	{
		edSource->setText(list.at(i));
		normalizeChars(edSource->editor());
		list[i] = edSource->text();
	} /* 			for(int i = 0; i < list.size(); ++i) */	
	edSource->setList(list);

	//         
	list = edAuthor->list();
	for(i = 0; i < list.size(); ++i)
	{
		edAuthor->setText(list.at(i));
		normalizeChars(edAuthor->editor());
		list[i] = edAuthor->text();
	} /* 			for(int i = 0; i < list.size(); ++i) */	
	edAuthor->setList(list);
	
	setPars(edQuest->editor());
	setPars(edAComm->editor());

	parentWidget()->setWindowModified(true);
	isDirty = true;
} /* QE_CentralWidget::normalize() */
