#include <QApplication>
#include <QDesktopWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTableWidget>
#include <QPushButton>
#include <QStringList>
#include <QInputDialog>
#include <QMessageBox>
#include <QHeaderView>

#include "sortdlg.h"
#include "tournament.h"
#include "quest.h"
#include "images/button_cancel.xpm"
#include "images/button_ok.xpm"
#include "images/sort.xpm"

/**
* \brief 
* \params aTourn -   
*/
QE_SortDlg::QE_SortDlg(QWidget *parent, QE_Tournament *aTourn):
	QDialog(parent),
	mTourn(aTourn)
{
	QVBoxLayout *topl = new QVBoxLayout(this);
	table = new QTableWidget( mTourn->countQuests(), 1, this);
	table->setHorizontalHeaderLabels(QStringList(tr("Text of question")));
	//table->setDragEnabled(true);
	topl->addWidget(table);
	
	QHBoxLayout *hb = new QHBoxLayout;
	hb->setSpacing(2);
	topl->addItem(hb);
	QPushButton *pb = new QPushButton(QIcon(sort_xpm), tr("&Move"), this);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotMove()));
	
	pb = new QPushButton(QIcon(button_ok), tr("Ok"), this);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotOk()));
	
	pb = new QPushButton(QIcon(button_cancel), tr("&Cancel"), this);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(reject()));

	setSizeGripEnabled(true);
	setMaximumHeight((QApplication::desktop()->height()*9)/10);
	showMaximized();
	fillTable();
	//table->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
} /* QE_SortDlg::*parent, QE_Tournament *aTourn) */

/**
* \brief   
*/
void
QE_SortDlg::fillTable()
{
	int i, count = mTourn->countQuests();
	QTableWidgetItem *it;
	for( i = 0; i < count; ++i)
	{
		it = new QTableWidgetItem();
		it->setData(0, mTourn->ptrQuest(i)->text());
		it->setData(1, i);
		table->setItem(i, 0, it);
		rowNums << tr("%1").arg(i + 1);
	} /* 	for( int i = 0; i < count; ++i) */
	for( i = 0; i < count; ++i)
		table->resizeRowToContents(i);
	
	table->setVerticalHeaderLabels(rowNums);
	//table->resizeColumnToContents(0);
	//table->horizontalHeader()->setStretchLastSection(true);
} /* QE_SortDlg::fillTable() */

/**
* \brief   
*       
*/
void
QE_SortDlg::slotMove()
{
	int curRow = table->currentRow();
	if ( curRow == -1)
	{
		QMessageBox::warning(this, tr("Attention!"),
				tr("You must select a qestion"));
		return;
	} /* 	if (table->currentRow() == -1) */
	
	bool ok;
	int qNum = QInputDialog::getInteger(this, tr("Entering of number"),
			tr("Enter number of quest"), 1, 1, mTourn->countQuests(), 1, &ok);
	if (ok && --qNum != curRow )
	{
		QTableWidgetItem *itCur = table->takeItem(curRow, 0);
		table->removeRow(curRow);
		table->insertRow(qNum);
		table->setItem(qNum, 0, itCur);
		table->setVerticalHeaderLabels(rowNums);
	} /* 	if (ok) */
} /* QE_SortDlg::slotMove() */

/**
* \brief   Ok.
*          .
*/
void
QE_SortDlg::slotOk()
{
	int newOder[mTourn->countQuests()];
	//[  
	for (int i = 0; i < table->rowCount(); ++i)
		newOder[i] = table->item(i, 0)->data(1).toInt();
	mTourn->reorder(newOder);
	accept();
} /* QE_SortDlg::slotOk() */


/**
* Resize event
*/
void
QE_SortDlg::resizeEvent(QResizeEvent *event)
{
	QDialog::resizeEvent(event);
	table->horizontalHeader()->resizeSection(0, childrenRect().width() - 
			(table->verticalHeader()->width() << 1));
} /* QE_SortDlg::resizeEvent(QResizeEvent *event) */
