#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QListWidget>
#include <QPushButton>
#include <QGroupBox>
#include <QTextCodec>
#include <QByteArray>

#include "spelldlg.h"

QE_SpellDlg::QE_SpellDlg(QE_CentralWidget *parent, const QString &title, 
						 AspellSpeller *sp,QTextEdit **edits, int cedits, 
						 const QColor &selColor, int numQuests):
	QE_TransWordDlg(parent, title, edits, cedits, selColor, numQuests),
	speller(sp),
	retCode(QDialog::Rejected)
{
	QVBoxLayout *topl = new QVBoxLayout(this);
	topl->setMargin(2);
	QHBoxLayout *hb = new QHBoxLayout;
	topl->addLayout(hb);
	hb->addWidget(new QLabel(tr("Unknown word:"), this));
	lbErrWord = new QLabel("ee", this);
	hb->addWidget(lbErrWord);

	hb = new QHBoxLayout;
	topl->addLayout(hb);
	QLabel *lb = new QLabel(tr("&Replace by"), this);
	hb->addWidget(lb);
	edReplace = new QLineEdit(this);
	hb->addWidget(edReplace);
	lb->setBuddy(edReplace);
	
	hb = new QHBoxLayout;
	topl->addLayout(hb);
	QVBoxLayout *vb = new QVBoxLayout;
	hb->addLayout(vb);
	lb = new QLabel(tr("&Variants"), this);
	vb->addWidget(lb);
	liVariants = new QListWidget(this);
	connect(liVariants, SIGNAL(itemSelectionChanged ()), this, SLOT(slotVariantChanged()));
	vb->addWidget(liVariants);
	lb->setBuddy(liVariants);
	
	vb = new QVBoxLayout;
	hb->addLayout(vb);
	QPushButton *pb = new QPushButton(tr("R&eplace"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotChange()));
	vb->addWidget(pb);
	pb = new QPushButton(tr("Re&place All"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotChangeAll()));
	vb->addWidget(pb);
	pb = new QPushButton(tr("&Skip"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotSkip()));
	vb->addWidget(pb);
	pb = new QPushButton(tr("S&kip ALL"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotSkipAll()));
	vb->addWidget(pb);
	QGroupBox *gb = new QGroupBox(this);
	gb->setFlat(true);
	vb->addWidget(gb);
	pb = new QPushButton(tr("&Add to dict"), this);
	vb->addWidget(pb);

	gb = new QGroupBox(this);
	topl->addWidget(gb);
	gb->setFlat(true);
	hb = new QHBoxLayout(gb);
	hb->setMargin(0);
	hb->addStretch(1);
	pb = new QPushButton(tr("&Close"), gb);
	pb->setDefault(true);
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotClose()));

	setSizeGripEnabled(true);

	codec = QTextCodec::codecForName("KOI8-R");
	nextError();
} /* QE_SpellDlg::*parent, const QString &title, AspellSpeller *sp, */

void
QE_SpellDlg::slotChange()
{
	retCode = QDialog::Accepted;
	clearSelection();
	replaceWord(edReplace->text());
	nextError();
} /* QE_SpellDlg::slotChange() */

void
QE_SpellDlg::slotChangeAll()
{
	retCode = QDialog::Accepted;
	clearSelection();
	allReplWords << word;
	rigthWords << edReplace->text();
	replaceWord(edReplace->text());
	nextError();
} /* QE_SpellDlg::slotChange() */

void
QE_SpellDlg::slotSkip()
{
	clearSelection();
	nextError();
} /* QE_SpellDlg::slotChange() */

void
QE_SpellDlg::slotSkipAll()
{
	clearSelection();
	skipedWords << word;
	nextError();
} /* QE_SpellDlg::slotChange() */

/**
* \brief    
*/
void
QE_SpellDlg::nextError()
{
	const char *koi8str;
	int ind;
	while(1)
	{
		nextWord();
		if (word.isEmpty())
		{
			slotClose();
			break;
		} /* 		if (word.isEmpty()) */
		koi8str = codec->fromUnicode(word).constData();
		if (!aspell_speller_check(speller, koi8str, -1))
		{
			ind = allReplWords.indexOf(word);
			if (ind != -1)
			{	//,    
				replaceWord(rigthWords[ind]);
				continue;	// . 
			} /*	if (ind != -1) */
			if (!skipedWords.contains(word, Qt::CaseInsensitive))
				break;
		} /* if (!aspell_speller_check(speller, koi8str, -1)) */
	} /* 	while(1) */
	lbErrWord->setText(word);
	selectCurWord();
	//  
	liVariants->clear();
	AspellWordList *suggestions = (AspellWordList*)aspell_speller_suggest(speller,
			koi8str, -1); 
	AspellStringEnumeration *elements = aspell_word_list_elements(suggestions); 
	while ((koi8str = aspell_string_enumeration_next(elements)) != NULL )
		liVariants->addItem(codec->toUnicode(koi8str));
	delete_aspell_string_enumeration(elements);
	
	if (liVariants->count())
		edReplace->setText(liVariants->item(0)->text());
} /* QE_SpellDlg::nextError() */

/**
* \brief   
*/
void
QE_SpellDlg::slotClose()
{
	if (! word.isEmpty())
		clearSelection();
	done(retCode);
} /* QE_SpellDlg::slotClose() */


/**
* \brief      
*/
void
QE_SpellDlg::slotVariantChanged()
{
	edReplace->setText(liVariants->currentItem()->text());
} /* QE_SpellDlg::slotVariantChanged() */
