#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPixmap>
#include <QTextEdit>
#include <QCheckBox>
#include <QPushButton>
#include <QSettings>

#include "toddlg.h"
#include "images/idea48.xpm"
#include "images/button_ok.xpm"
#include "images/1rightarrow.xpm"

/**
* \brief 
* \params aParent - -
* \params aTipFile -    
*/
QE_TODDlg::QE_TODDlg(QWidget *aParent, const QString &aTipFile, bool *aShowTip):
	QDialog(aParent), tipFile(aTipFile), pShowTip(aShowTip)
{
	setWindowTitle(tr("Tip of the day"));

	QVBoxLayout *topLayout = new QVBoxLayout(this);

	QHBoxLayout *hb = new QHBoxLayout;
	QLabel *lb = new QLabel(this);
	lb->setPixmap(QPixmap(idea48));
	hb->addWidget(lb);
	hb->addWidget(new QLabel(tr("Did you know...?"), this));
	topLayout->addLayout(hb);

	edTip = new QTextEdit(this);
	edTip->setReadOnly(true);
	topLayout->addWidget(edTip);

	hb = new QHBoxLayout;
	cbShowTip = new QCheckBox(tr("Show tips at next start"), this);
	cbShowTip->setCheckState(*pShowTip ? Qt::Checked : Qt::Unchecked);
	hb->addWidget(cbShowTip);
	QPushButton *pb = new QPushButton(QIcon(i1rightarrow), tr("&Next tip"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotNext()));
	hb->addWidget(pb);
	pb = new QPushButton(QIcon(button_ok), tr("&Close tips"), this);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotClose()));
	hb->addWidget(pb);
	topLayout->addLayout(hb);

	slotNext();
} /* QE_TODDlg::*aParent, const QString &aTipFile) */


/**
* \brief slotClose
*
*   .      
*  .
*/
void
QE_TODDlg::slotClose()
{
	*pShowTip = cbShowTip->checkState() == Qt::Checked;
	accept();
} /* QE_TODDlg::slotClose() */


/**
* \brief   next
*/
void
QE_TODDlg::slotNext()
{
	QSettings tips(tipFile, QSettings::IniFormat);
	int curTip = tips.value("nexttip", 0).toInt();
	int countTip = tips.value("count", 0).toInt();
	if( curTip >= countTip)
		curTip = 0;
	edTip->setHtml(QString::fromUtf8(tips.value(QString("tip") + 
					QString::number(curTip)).toByteArray().constData()));
	tips.setValue("nexttip", ++curTip);
} /* QE_TODDlg::slotNext() */
