#include <QFile>
#include <QMap>
#include <QMessageBox>
#include <QObject>

#include "tournament.h"
#include "XMLWriter.h"
#include "quest.h"
#include "qedreader.h"

QE_Tournament::QE_Tournament()
{
	mTournInfoData.cTurQuests = 0;
} /* QE_Tournament() */

void
QE_Tournament::setTournamentInfo(const QE_TournInfoData &aTourInfo)
{
	mTournInfoData.title = aTourInfo.title;
	mTournInfoData.description = aTourInfo.description;
	mTournInfoData.day = aTourInfo.day;
	mTournInfoData.month = aTourInfo.month;
	mTournInfoData.year = aTourInfo.year;
	mTournInfoData.cTurQuests = aTourInfo.cTurQuests;
} /* QE_Tournament::setTournamentInfo(const QE_TourInfoData &tourInfo) */

void
QE_Tournament::getTournamentInfo(QE_TournInfoData *aTourInfo) const
{
	aTourInfo->title = mTournInfoData.title;
	aTourInfo->description = mTournInfoData.description;
	aTourInfo->day = mTournInfoData.day;
	aTourInfo->month = mTournInfoData.month;
	aTourInfo->year = mTournInfoData.year;
	aTourInfo->cTurQuests = mTournInfoData.cTurQuests;
} /* QE_Tournament::getTournamentInfo(QE_TourInfoData *tourInfo) const */

/**
* \brief    
* \param aFileName -  
* \returns true  [, false -  
*/
bool
QE_Tournament::save(const QString &aFileName) const
{
	QMap <QString, QString> attrs;
	QFile file(aFileName);
	file.open(QIODevice::WriteOnly);
	XMLWriter writer(&file);
	
	writer.writeOpenTag("tournament");
	
	writer.writeOpenTag("info");
	attrs.insert("sign", "CHGK");
	attrs.insert("title", mTournInfoData.title);
	attrs.insert("description", mTournInfoData.description);
	attrs.insert("date", QString("%1-%2-%3").arg(mTournInfoData.day)
									   .arg(mTournInfoData.month)
									   .arg(mTournInfoData.year));
	QString turquests;
	turquests.setNum(mTournInfoData.cTurQuests)	;
	attrs.insert("turquests", turquests);
	writer.writeAtomTag("metainfo", &attrs);
	// 
	for(QVector<QE_EditorInfo>::const_iterator it = mTournInfoData.editors.constBegin();
			it != mTournInfoData.editors.constEnd(); ++it )
	{
		attrs.clear();
		attrs.insert("name", it->name);
		attrs.insert("email", it->email);
		writer.writeAtomTag("editor", &attrs);
	} /* it != mTournInfoData.constEnd(); ++it ) */
	writer.writeCloseTag("info");
	
	writer.writeOpenTag("quests");
	for(QVector<QE_Quest*>::const_iterator it = mQuests.constBegin();
			it != mQuests.constEnd(); ++it)
		(*it)->save(writer);
	
	writer.writeCloseTag("quests");
	writer.writeCloseTag("tournament");
	return true;
} /* QE_Tournament::save(const QString aFileName) const */


/**
* \brief    
* \param aFileName -  
* \returns true  [, false -  
*/
bool
QE_Tournament::load(const QString &aFileName)
{
	QE_QuestReader handler(this);
	QXmlSimpleReader reader;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	
	QFile file(aFileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0, QObject::tr("quested"),
                             QObject::tr("Cannot read file %1:\n%2.")
                             .arg(aFileName)
                             .arg(file.errorString()));
        return false;
    }

    QXmlInputSource xmlInputSource(&file);
    return reader.parse(xmlInputSource);
} /* QE_Tournament::load(const QString &aFileName) */

/**
*    :     
*/
void
QE_Tournament::clear()
{
	mTournInfoData.title.clear();
	mTournInfoData.description.clear();
	mTournInfoData.day = 0;
	mTournInfoData.month = 0;
	mTournInfoData.year = 0;
	mTournInfoData.cTurQuests = 0;
	mTournInfoData.editors.clear();

	int i = mQuests.size() - 1;
	while (!mQuests.isEmpty())
	{
		delete mQuests.last();
		mQuests.remove(i--);
	} /* 	while (!mQuests.isEmpty()) */
} /* QE_Tournament::clear() */

/**
* \brief    
* \param newOrder -      
*/
void
QE_Tournament::reorder(int *newOrder)
{
	QVector<QE_Quest*> tempArr(mQuests);
	int i, cq = countQuests();
	for( i = 0; i < cq; ++i)
		tempArr.append(mQuests[i]);
	mQuests.clear();
	for( i = 0; i < cq ; ++i)
		mQuests.append(tempArr[newOrder[i]]);
	tempArr.clear();
} /* QE_Tournament::reorder(int *newOrder) */

/**
* \brief     
*/
QSet<QString>
QE_Tournament::authors() const
{
	QSet<QString> auts;
	for(int i = 0; i < countQuests(); ++i)
		auts.insert((mQuests[i])->author());
	return auts;
} /* QE_Tournament::authors() const */

/**
* \brief    
*
*     
* \param aQuestNum -   
* \warning    
*/
void
QE_Tournament::removeQuest(int aQuestNum)
{
	//QE_Quest *p = mQuests[aQuestNum];
	mQuests.remove(aQuestNum);
	//delete p;
} /* QE_Tournament::removeQuest(int aQuestNum) */
