#include <QLineEdit>
#include <QDateEdit>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGroupBox>
#include <QLabel>
#include <QSpinBox>
#include <QPushButton>
#include <QTableWidget>
#include <QMessageBox>

#include "tourninfodlg.h"
#include "editordlg.h"
#include "tournament.h"

#include "images/button_ok.xpm"
#include "images/button_cancel.xpm"

/**
* 
* \param aTournInfo -   ,    
*  
*/
QE_TournInfoDialog::QE_TournInfoDialog(QE_TournInfoData *aTournInfo,
	   	QWidget *parent):
	QDialog(parent), tournInfo(aTournInfo)
{
	setWindowTitle(tr("Tournament's information"));
	QVBoxLayout *topl = new QVBoxLayout(this);
	topl->setSpacing(2);
	topl->setMargin(2);
	
	QLabel *lb = new QLabel(tr("&Name of tournament"), this);
	topl->addWidget(lb);
	edTitle = new QLineEdit(tournInfo->title, this);
	lb->setBuddy(edTitle);
	topl->addWidget(edTitle);
	
	lb = new QLabel(tr("&Description of tournament"), this);
	topl->addWidget(lb);
	edDescr = new QLineEdit(tournInfo->description, this);
	lb->setBuddy(edDescr);
	topl->addWidget(edDescr);
	
	QGroupBox *gb = new QGroupBox(tr("&Editors of package"), this);
	topl->addWidget(gb);
	QVBoxLayout *vb = new QVBoxLayout(gb);
	twEditors = new QTableWidget(0, 2, gb);
	vb->addWidget(twEditors);
	QStringList sl(tr("Name"));
	sl << tr("@ E-mail");
	twEditors->setHorizontalHeaderLabels(sl);
	QHBoxLayout *hb = new QHBoxLayout;
	hb->setSpacing(5);
	hb->addStretch(1);
	vb->addItem(hb);
	QPushButton *pb = new QPushButton(tr("&Add"), gb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotAdd()));
	hb->addWidget(pb);
	pb = new QPushButton(tr("Dele&te"), gb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotDel()));
	hb->addWidget(pb);
	
	lb = new QLabel(tr("Dat&e of tournament"), this);
	topl->addWidget(lb);
	deDate = new QDateEdit(QDate(tournInfo->year, tournInfo->month, tournInfo->day), this);
	deDate->setDisplayFormat("dd-MMM-yyyy");
	topl->addWidget(deDate);
	lb->setBuddy(deDate);
	
	hb = new QHBoxLayout;
	topl->addItem(hb);
	sbCQuests = new QSpinBox(this);
	sbCQuests->setRange(0, 30);
	hb->addWidget(sbCQuests);
	hb->addSpacing(5);
	lb = new QLabel(tr("&Questions in tour"), this);
	hb->addWidget(lb, 1);
	lb->setBuddy(sbCQuests);

	//buttons
	gb = new QGroupBox(this);
	topl->addWidget(gb);
	gb->setFlat(true);
	hb = new QHBoxLayout(gb);
	//hb->setMargin(2);
	hb->addStretch(1);
	pb = new QPushButton(tr("Ok"), gb);
	pb->setIcon(QIcon(button_ok));
	hb->addWidget(pb);
	pb->setDefault(true);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(slotOk()));
	pb = new QPushButton(tr("Cancel"), gb);
	pb->setIcon(QIcon(button_cancel));
	hb->addWidget(pb);
	connect(pb, SIGNAL(clicked(bool)), this, SLOT(reject()));
	
	setLayout(topl);
	setSizeGripEnabled(true);

	edTitle->setText(tournInfo->title);
	edDescr->setText(tournInfo->description);
	//deDate->setDate(QDate::fromString(tournInfo->date, "dd-MMM-yyyy"));
	sbCQuests->setValue(tournInfo->cTurQuests);
	for(QVector<QE_EditorInfo>::const_iterator it = tournInfo->editors.constBegin();
			it != tournInfo->editors.constEnd(); ++it )
	{
		int row = twEditors->rowCount();
		twEditors->insertRow(row);
		twEditors->setItem(row, 0, new QTableWidgetItem(it->name));
		twEditors->setItem(row, 1, new QTableWidgetItem(it->email));
	} /* it != tournInfo->editors.constEnd(); ++it ) */
} /* QE_TournInfoDialog::*aTournInfo, QWidget *parent) */

void
QE_TournInfoDialog::slotOk()
{
	QE_EditorInfo ei;
	tournInfo->title = edTitle->text();
	tournInfo->description = edDescr->text();
	//tournInfo->date = edDate->text();
	//tournInfo->date = deDate->text();
	QDate date = deDate->date();
	tournInfo->year = date.year();
	tournInfo->month = date.month();
	tournInfo->day = date.day();
	tournInfo->cTurQuests = sbCQuests->value();
	int countRow = twEditors->rowCount();
	tournInfo->clearEditors();
	for(int i = 0; i < countRow; ++i)
	{
		ei.name  = twEditors->item(i, 0)->text();
		ei.email = 	twEditors->item(i, 1)->text();
		tournInfo->addEditorInfo(ei);
	} /* 	for(int i = 0; i < countRow; ++i) */
	accept();
} /* QE_TournInfoDialog::slotOk() */

void
QE_TournInfoDialog::slotAdd()
{
	QE_EditorInfo ei;
	QE_EditorDialog dlg(&ei, this);
	if(dlg.exec() == QDialog::Accepted && !ei.name.isEmpty())
	{
		int row = twEditors->rowCount();
		twEditors->insertRow(row);
		twEditors->setItem(row, 0, new QTableWidgetItem(ei.name));
		twEditors->setItem(row, 1, new QTableWidgetItem(ei.email));
	} /* 	if(dlg.exec() == QDialog::Accepted) */
} /* QE_TournInfoDialog::slotAdd() */

/**
*  Del
*/
void
QE_TournInfoDialog::slotDel()
{
	int row = twEditors->currentRow();
	if( row >= 0)
		twEditors->removeRow(row);
	else
		QMessageBox::warning(this, tr("Warning"), 
				tr("You should select an editor fo deleting"));
} /* QE_TournInfoDialog::slotDel() */
